/* 
 *   Copyright (c) 2000 Michael J. Roberts
 *   
 *   This file is part of TADS 3.  
 */

#ifndef _ARRAY_H_
#define _ARRAY_H_

/*
 *   'array' metaclass 
 */
intrinsic class Array 'array/030001'
{
    /* 
     *   Create a list with the same elements as the array.  If 'start' is
     *   specified, it's the index of the first element we store; we'll
     *   store elements starting at index 'start'.  If 'cnt' is specified,
     *   it gives the maximum number of elements for the new list; by
     *   default, we'll store all of the elements from 'start' to the last
     *   element.  
     */
    toList(start?, cnt?);

    /* get the number of elements in the array */
    length();

    /* 
     *   Copy from another array or list.  Elements are copied from the
     *   source array or list starting at the element given by
     *   'src_start', and are copied into 'self' starting at the index
     *   given by 'dst_start'.  At most 'cnt' values are copied, but we
     *   stop when we reach the last element of either the source or
     *   destination values.  
     */
    copyFrom(src, src_start, dst_start, cnt);

    /* 
     *   fill with a given value, starting at the given element (the first
     *   element if not specified), and running for the given number of
     *   elements (the remaining elements of the array, if not not
     *   specified) 
     */
    fillValue(val, start?, cnt?);

    /*
     *   Select a subset of the array.  Returns a new array consisting
     *   only of the elements of this array for which the callback
     *   function returns true.  
     */
    subset(func);

    /*
     *   Apply a callback function to each element of the array.  For each
     *   element of the array, invokes the callback, and replaces the
     *   element with the return value of the callback.  Modifies the
     *   array in-place, and returns 'self'. 
     */
    applyAll(func);
}

#endif /* _ARRAY_H_ */

