/*
 *   Copyright 2000 Michael J. Roberts.
 *   
 *   This file is part of TADS 3. 
 */


/*
 *   TADS datatype codes.  These values are returned by propType(), etc.  
 */
#define TYPE_NIL         1
#define TYPE_TRUE        2
#define TYPE_OBJECT      5
#define TYPE_PROP        6
#define TYPE_INT         7
#define TYPE_SSTRING     8
#define TYPE_DSTRING     9
#define TYPE_LIST        10
#define TYPE_CODE        11
#define TYPE_FUNCPTR     12
#define TYPE_NATIVE_CODE 14
#define TYPE_ENUM        15


/*
 *   The root object class.  All objects descend from this class. 
 */
intrinsic class Object 'root-object/030001'
{
    /* determine if I'm an instance of the given class 'cls' */
    isClass(cls);

    /* get the list of direct superclasses of this object */
    getSuperclassList();

    /* determine if a property is defined for this object */
    propDefined(prop, flags?);

    /* get the type of a property defined for this object */
    propType(prop);
}

/*
 *   propDefined() flags 
 */
#define PROPDEF_ANY           1
#define PROPDEF_DIRECTLY      2
#define PROPDEF_INHERITS      3
#define PROPDEF_GET_CLASS     4

/*
 *   The IntrinsicClass intrinsic class 
 */
intrinsic class IntrinsicClass 'intrinsic-class/030000'
{
}

/*
 *   Anonymous function pointer intrinsic class 
 */
intrinsic class AnonFuncPtr 'anon-func-ptr'
{
}

/*
 *   The "TADS Object" intrinsic class.  All objects that the program
 *   defines with the "class" or "object" statements descend from this
 *   class.  
 */
intrinsic class TadsObject 'tads-object/030000'
{
}

/*
 *   The native string type. 
 */
intrinsic class String 'string/030001'
{
    /* get the length of the string */
    length();

    /* extract a substring */
    substr(start, len?);

    /* convert to upper case */
    toUpper();

    /* convert to lower case */
    toLower();

    /* find a substring */
    find(str);

    /* 
     *   convert to a list of Unicode character codes, or get the Unicode
     *   character code for the single character at the given index 
     */
    toUnicode(idx?);
}

/*
 *   The native list type 
 */
intrinsic class List 'list/030001'
{
    /* 
     *   Select a subset of the list: returns a new list consisting only
     *   of the elements for which the callback function 'func' returns
     *   true.  
     */
    subset(func);

    /*
     *   Apply the callback function to each element of this list, and
     *   return a new list consisting of the results 
     */
    applyAll(func);

    /* get the number of elements in the list */
    length();

    /* extract a sublist */
    sublist(start, len?);

    /* intersect with another list */
    intersect(other);

    /* find a value */
    find(val);

    /* car/cdr - head/tail of list */
    car();
    cdr();
}

